﻿using Microsoft.CSharp;
using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web.Razor;

namespace Web
{
    /// <summary>
    /// Razor引擎
    /// </summary>
    public class RazorEngine
    {
        private string outputAssemblyName = "RazorOutput.dll";
        private string defaultClassName = "CustmerTemplate";
        private string baseClassFullName = typeof(TemplateBase).FullName;
        private string defaultNamespace = "RazorOutput";
        private string tempStr = string.Empty;
        private GeneratorResults generatorResult = null;
        /// <summary>
        /// 模板字符串
        /// </summary>
        public string TempStr
        {
            get { return tempStr; }
            set { tempStr = value; }
        }
        /// <summary>
        /// 类名称
        /// </summary>
        public string DefaultClassName
        {
            get { return defaultClassName; }
            set { defaultClassName = value; }
        }
        /// <summary>
        /// 基类名称
        /// </summary>
        public string BaseClassFullName
        {
            get { return baseClassFullName; }
            set { baseClassFullName = value; }
        }
        /// <summary>
        /// 命名空间
        /// </summary>
        public string DefaultNamespace
        {
            get { return defaultNamespace; }
            set { defaultNamespace = value; }
        }
        /// <summary>
        /// 程序集名称
        /// </summary>
        public string OutputAssemblyName
        {
            get { return outputAssemblyName; }
            set { outputAssemblyName = value; }
        }

        public RazorEngine(string _tempStr)
        {
            tempStr = _tempStr;
        }

        /// <summary>
        /// 解析后的.net代码
        /// </summary>
        /// <returns></returns>
        public string GetGeneratorCode()
        {
            //1.创建RazorEngineHost--------------------------------------------------------------
            var razorEngineHost = CreateRazorEngineHost(DefaultClassName, BaseClassFullName, DefaultNamespace);
            //2.创建RazorTemplateEngine
            var razorTemplateEngine = new RazorTemplateEngine(razorEngineHost);
            //3.解析模板到.Net代码
            generatorResult = GetTemplateGeneratorResult(razorTemplateEngine, tempStr);
            //4.查看对应的.Net代码
            var generatorResultStr = GetTemplateGeneratorResultStr(generatorResult);
            return generatorResultStr;
        }

        /// <summary>
        /// 执行后的模板内容
        /// </summary>
        /// <returns></returns>
        public string Render()
        {
            if (null == generatorResult)
            {
                //1.创建RazorEngineHost--------------------------------------------------------------
                var razorEngineHost = CreateRazorEngineHost(DefaultClassName, BaseClassFullName, DefaultNamespace);
                //2.创建RazorTemplateEngine
                var razorTemplateEngine = new RazorTemplateEngine(razorEngineHost);
                //3.解析模板到.Net代码
                generatorResult = GetTemplateGeneratorResult(razorTemplateEngine, tempStr);
            }
            //4.编译.Net代码到类库
            var compilerResult = GetCompilerResult(generatorResult, OutputAssemblyName);
            if (!compilerResult.Errors.HasErrors)
            {
                //5.执行模板结果
                var templateResult = GetRazorTemplateResult(OutputAssemblyName, DefaultClassName, DefaultNamespace);
                return templateResult;
            }
            else
            {
                throw new Exception("编译执行出现问题");
            }
        }
        //创建RazorEngineHost
        static RazorEngineHost CreateRazorEngineHost(string className, string baseClassFullName, string defaultNamespace)
        {
            var language = new CSharpRazorCodeLanguage();
            var razorEngineHost = new RazorEngineHost(language);
            //设置基类名称
            razorEngineHost.DefaultBaseClass = baseClassFullName;
            //设置类名称
            razorEngineHost.DefaultClassName = className;
            //设置命名空间
            razorEngineHost.DefaultNamespace = defaultNamespace;
            //设置引用的程序集
            razorEngineHost.NamespaceImports.Add("System");

            return razorEngineHost;
        }
        //解析模板到.Net代码
        protected GeneratorResults GetTemplateGeneratorResult(RazorTemplateEngine razorEngine, string TemplateStr)
        {
            try
            {
                GeneratorResults razorResult = null;
                using (TextReader textReader = new StringReader(TemplateStr))
                {
                    razorResult = razorEngine.GenerateCode(textReader);
                }
                return razorResult;
            }
            catch (Exception)
            {

                throw;
            }

        }
        //查看对应的.Net代码
        protected string GetTemplateGeneratorResultStr(GeneratorResults generatorResult)
        {
            var outputStr = string.Empty;
            var codeProvider = new CSharpCodeProvider();
            using (StringWriter sw = new StringWriter())
            {
                codeProvider.GenerateCodeFromCompileUnit(generatorResult.GeneratedCode, sw, new CodeGeneratorOptions());
                outputStr = sw.GetStringBuilder().ToString();
            }
            return outputStr;
        }
        //编译.Net代码到类库
        protected CompilerResults GetCompilerResult(GeneratorResults generatorResult, string outputAssemblyName)
        {
            var codeProvider = new CSharpCodeProvider();
            var baseAssembly = typeof(Program).Assembly.CodeBase.Replace("file:///", "").Replace("/", "\\");
            var compileParams = new CompilerParameters(new string[]{
                baseAssembly
            }, outputAssemblyName);
            // 编译生成的代码到程序集
            return codeProvider.CompileAssemblyFromDom(compileParams, generatorResult.GeneratedCode);
        }
        //得到执行结果
        protected string GetRazorTemplateResult(string outputAssemblyName, string className, string nameSpace)
        {
            // 加载程序集
            var assembly = Assembly.LoadFrom(outputAssemblyName);
            var type = assembly.GetType(nameSpace + "." + className);
            var template = Activator.CreateInstance(type) as TemplateBase;
            template.Execute();
            return template.Render();
        }
    }
}
